local Skada = Skada

Skada:AddLoadableModule("Healing", function(Skada, L)
	if Skada.db.profile.modulesBlocked.Healing then return end
	
	local mod = Skada:NewModule("Multistrike healing")
	local playerMulti = Skada:NewModule("Multistrike healing spell list")
	local spellsMulti = Skada:NewModule("Multistrike healing spell details")
	local multiTarget = Skada:NewModule("Multistrike healing targets")

	local sourceHeal
	local function getMultiHPS(set, player, sourceHeal)
		local totaltime = Skada:PlayerActiveTime(set, player)

		return sourceHeal / math.max(1, totaltime)
	end

	local function getRaidMultiHPS(set)
		if set.time > 0 then
			return set.multiHealing / math.max(1, set.time)
		else
			local endtime = set.endtime
			if not endtime then
				endtime = time()
			end
			return set.multiHealing / math.max(1, endtime - set.starttime)
		end
	end

	local function log_healing(set, multi)
		-- Get the player.
		local player = Skada:get_player(set, multi.playerid, multi.playername)
		if player then

			-- Subtract overkill
	--		local amount = math.max(0,multi.amount - multi.overkill)
			-- Or don't. Seems to be the way other meters do it.
			local amount = multi.amount
	--		self:Print(player.name..": "..multi.spellname.." for "..tostring(amount))

			-- Also add to set total and total non multistrike damage. NOT USED
			-- set.nonMultiHealing = set.nonMultiHealing + amount		

			-- Add spell to player if it does not exist.
			-- Add total multistrikes if not exist
			if not player.multiHealingSpells[multi.spellname] then
				player.multiHealingSpells[multi.spellname] = {id = multi.spellid, totalHitHeals = 0, healing = 0, totalMultiHeals = 0, multiHealing = 0}
			end
			
			-- Get the spell from player.
			local spell = player.multiHealingSpells[multi.spellname]
			
			-- Add to player total healing.
			player.nonMultiHealing = player.nonMultiHealing + amount
			spell.healing = spell.healing + amount
			
			player.totalHitHeals = player.totalHitHeals + 1
			spell.totalHitHeals = spell.totalHitHeals + 1
			
			if multi.multistrike then
				spell.totalMultiHeals = spell.totalMultiHeals + 1
				spell.multiHealing = spell.multiHealing + amount
				
				player.totalMultiHeals = player.totalMultiHeals + 1
				player.multiHealing = player.multiHealing + amount
				
				set.multiHealing = set.multiHealing + amount
				
				if spell.max == nil or amount > spell.max then
					spell.max = amount
				end
				
				if (spell.min == nil or amount < spell.min) and not multi.missed then
					spell.min = amount
				end
			
				do
					if multi.dstName and multi.dstGUID then
						local multiHealed = player.multiHealed[multi.dstGUID]

						-- Create recipient if it does not exist.
						if not multiHealed then
							local _, className = UnitClass(multi.dstName)
							local playerRole = UnitGroupRolesAssigned(multi.dstName)
							multiHealed = {name = multi.dstName, class = className, role = playerRole, amount = 0}
							player.multiHealed[multi.dstGUID] = multiHealed
						end

						multiHealed.amount = multiHealed.amount + amount
					end
				end
			end
		end
	end
		
	local multi = {}

	local function SpellHeal(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
		-- Healing
		local spellId, spellName, spellSchool, samount, soverhealing, absorbed, scritical, smultistrike = ...	-- Spell healing.
		
		if bit.band(srcFlags, COMBATLOG_OBJECT_CONTROL_PLAYER) ~= 0 and bit.band(srcFlags, dstFlags, COMBATLOG_OBJECT_REACTION_MASK) ~= 0 then
			multi.dstName = dstName
			multi.dstGUID = dstGUID
			multi.playerid = srcGUID
			multi.playername = srcName
			multi.spellid = spellId
			multi.spellname = spellName
			multi.amount = samount
			multi.multistrike = smultistrike
			
			Skada:FixPets(multi)
			log_healing(Skada.current, multi)
			log_healing(Skada.total, multi)
		end
	end

	function mod:Update(win, set)
		local max = 0
		local nr = 1
		
		for i, player in ipairs(set.players) do
			if player.nonMultiHealing > 0 then
				local multiHPS = getMultiHPS(set, player, player.multiHealing)

				local d = win.dataset[nr] or {}
				win.dataset[nr] = d
				d.label = player.name

				d.value = player.multiHealing
				d.id = player.id
				d.class = player.class
				d.role = player.role
				
				d.valuetext = Skada:FormatValueText(
					Skada:FormatNumber(player.multiHealing), self.metadata.columns.Healing,
					Skada:FormatNumber(multiHPS), self.metadata.columns.HPS,
					string.format("%02.1f%%", (player.multiHealing / set.multiHealing) * 100), self.metadata.columns.Percent
				)
				if player.multiHealing > max then
					max = player.multiHealing
				end
				nr = nr + 1
			end
		end

		win.metadata.maxvalue = max
	end
	
	local function player_tooltip(win, id, label, tooltip)
		local player = Skada:find_player(win:get_selected_set(), playerMulti.playerid)
		if player then
			local spell = player.multiHealingSpells[label]
			if spell then
				tooltip:AddLine(player.name.." - "..label)
				if spell.max and spell.min then
					tooltip:AddDoubleLine(L["Minimum hit:"], Skada:FormatNumber(spell.min), 255,255,255,255,255,255)
					tooltip:AddDoubleLine(L["Maximum hit:"], Skada:FormatNumber(spell.max), 255,255,255,255,255,255)
				end
				tooltip:AddDoubleLine(L["Average hit:"], Skada:FormatNumber(spell.multiHealing / spell.totalMultiHeals), 255,255,255,255,255,255)
			end
		end
	end
	
	function playerMulti:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), id)
		if player then
			playerMulti.playerid = id
			playerMulti.title = player.name.."'s Multistrike Healing"
		end
	end

	function playerMulti:Update(win,set)
		-- View spells for this player.
		local player = Skada:find_player(set, self.playerid)
		local max = 0

		-- If we reset we have no data.
		if player then

			local nr = 1
			if player then
				for spellname, spell in pairs(player.multiHealingSpells) do

					local d = win.dataset[nr] or {}
					win.dataset[nr] = d
					d.label = spellname
					d.id = spellname
					local _, _, icon = GetSpellInfo(spell.id)
					d.icon = icon
					d.spellid = spell.id
					d.value = spell.multiHealing
					d.valuetext = Skada:FormatValueText(
						Skada:FormatNumber(spell.multiHealing), self.metadata.columns.Healing,
						string.format("%02.1f%%", (spell.multiHealing / player.multiHealing) * 100), self.metadata.columns.Percent
					)
					if spell.multiHealing > max then
						max = spell.multiHealing
					end
					nr = nr + 1
				end
			end
		end

		win.metadata.maxvalue = max
	end
	
	local function add_detail_bar(win, title, value)
		local nr = spellsMulti.nr + 1
		spellsMulti.nr = nr
		local d = win.dataset[nr] or {}
		win.dataset[nr] = d

		d.value = value
		d.label = title
		d.id = title
		d.valuetext = Skada:FormatValueText(
			value, mod.metadata.columns.Healing,
			string.format("%02.1f%%", value / spellsMulti.totalHitHeals * 100), mod.metadata.columns.Percent
		)

		win.metadata.maxvalue = math.max(win.metadata.maxvalue, value)
	end
	
	function spellsMulti:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), playerMulti.playerid)
		spellsMulti.spellname = label
		spellsMulti.title = player.name.."'s "..label
	end
	--
	function spellsMulti:Update(win, set)
		local player = Skada:find_player(set, playerMulti.playerid)

		if player then
		local spell = player.multiHealingSpells[self.spellname]

			if spell then
				spellsMulti.totalHitHeals = spell.totalHitHeals
				spellsMulti.nr = 0
				win.metadata.maxvalue = 0
				
				if spell.totalHitHeals and spell.totalHitHeals > 0 then
					add_detail_bar(win, "Total", spell.totalHitHeals)
				end
				if spell.totalMultiHeals and spell.totalMultiHeals > 0 then
					add_detail_bar(win, "Multistrike", spell.totalMultiHeals)
				end
			end
		end
	end

	function multiTarget:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), id)
		multiTarget.playerid = id
		multiTarget.title = (player and player.name or "Unknown").."'s Multistrike healed targets"

	end
--
	function multiTarget:Update(win, set)
		local player = Skada:find_player(set, self.playerid)
		local max = 0
		local nr = 1
		
		if player then
			for dstGUID, multiHealed in pairs(player.multiHealed) do
				if multiHealed.amount > 0 then

					local d = win.dataset[nr] or {}
					win.dataset[nr] = d

					d.id = dstGUID
					d.label = multiHealed.name or dstGUID
					d.value = multiHealed.amount
					d.class = multiHealed.class
                    d.role = player.role
					d.valuetext = Skada:FormatValueText(
						Skada:FormatNumber(multiHealed.amount), self.metadata.columns.Healing,
						string.format("%02.1f%%", multiHealed.amount / player.multiHealing * 100), self.metadata.columns.Percent
						)
					if multiHealed.amount > max then
						max = multiHealed.amount
					end

					nr = nr + 1
				end
			end
		end

		win.metadata.maxvalue = max
		win.metadata.hasicon = true
	end

	function mod:OnEnable()
		mod.metadata = {showspots = true, click1 = playerMulti, click2 = multiTarget, columns = {Healing = true, HPS = true, Percent = true}}
		playerMulti.metadata	= {tooltip = player_tooltip, click1 = spellsMulti, columns = {Healing = true, Percent = true}}
		spellsMulti.metadata = {columns = {Healing = true, Percent = true}}
		multiTarget.metadata = {showspots = true, columns = {Healing = true, Percent = true}}
			

		Skada:RegisterForCL(SpellHeal, 'SPELL_HEAL', {src_is_interesting = true})
		Skada:RegisterForCL(SpellHeal, 'SPELL_PERIODIC_HEAL', {src_is_interesting = true})
		
		Skada:AddMode(self)
	end

	function mod:OnDisable()
		Skada:RemoveMode(self)
	end

	function mod:AddPlayerAttributes(player)
		if not player.multiHealingSpells or not player.multiHealing or not player.totalMultiHeals or not player.totalHitHeals or not player.nonMultiHealing then
			player.multiHealingSpells = player.multiHealingSpells or {}		-- Stored spell multistrike info
			player.multiHealing = player.multiHealing or 0					-- Total multistrike healing
			player.totalMultiHeals = player.totalMultiHeals or 0
			player.totalHitHeals = player.totalHitHeals or 0
			player.nonMultiHealing = player.nonMultiHealing or 0
		end
		
		if not player.multiHealed then
			player.multiHealed = {}
		end
	end

	function mod:AddToTooltip(set, tooltip)
		GameTooltip:AddDoubleLine("Multistrike HPS", Skada:FormatNumber(getRaidMultiHPS(set)), 1,1,1)
	end

	function mod:AddSetAttributes(set)
		if not set.nonMultiHealing then
			set.nonMultiHealing = 0
		end
		if not set.multiHealing then
			set.multiHealing = 0
		end
	end

	function mod:GetSetSummary(set)
		return Skada:FormatValueText(
			Skada:FormatNumber(set.multiHealing), self.metadata.columns.Healing,
			Skada:FormatNumber(getRaidMultiHPS(set)), self.metadata.columns.HPS
		)
	end
end)